let ed_utils = {
    categoriesHierarchy: ["ed", "domains", "levels", "properties", "values", "emotions"],
    typesHierarchy: ["ed", "domain", "level", "property", "value", "emotion"],
    getFromCategory: (category, labels) => {
        let r = {};
        if (typeof labels === "string") labels = this.labelsStringToArray(labels);
        for (let l of labels)
            if (category[l]) r[l] = category[l];
        return r;
    },
    labelsStringToArray: labelsStr => {
        if (typeof labelsStr === "string") labelsStr = labelsStr.replace(/\s/g, '').split(",");
        return labelsStr === "" ? [] : labelsStr
    },
}

class BaseEDModule {
    constructor(label) {
        this.allLabels = label.replace(/\s/g, '').split("/");
        this.label = this.allLabels[0];
        this.otherlabels = this.allLabels.slice(1);
        this.category = "";
        this.type = "";
        this.parents = {};
        this.children = {};
        this.parentsCategory = "";
        this.parentsType = "";
        this.childrenCategory = "";
        this.childrenType = ""
        this.ed = null;
        this.childrenLabels = [];
    }

    randomChild() {
        let childrenArr = Object.values(this.children);
        let index = Math.floor(Math.random() * childrenArr.length);
        return childrenArr[index];
    }

    randomParent() {
        let parentsArr = Object.values(this.parents);
        let index = Math.floor(Math.random() * parentsArr.length);
        return parentsArr[index];
    }

    setCatsAndTypes() {
        let hindex = ed_utils.categoriesHierarchy.indexOf(this.category);
        this.parentsCategory = ed_utils.categoriesHierarchy[hindex - 1];
        this.parentsType = ed_utils.typesHierarchy[hindex - 1];
        this.childrenCategory = ed_utils.categoriesHierarchy[hindex + 1];
        this.childrenType = ed_utils.typesHierarchy[hindex + 1];
    }

    setChildrenFromChildrenLabels() {
        this[this.childrenCategory] = ed_utils.getFromCategory(this.ed[this.childrenCategory], this.childrenLabels);

        let chs = this[this.childrenCategory];
        for (let k of Object.keys(chs)) {
            this.children[k] = chs[k];
            chs[k].setParent(this);
            chs[k].setChildrenFromChildrenLabels();
        }
        return this.children;
    }

    setParent(parent) {
        this.parents[parent.label] = parent;
        if (!this.ed) this.ed = parent.ed;
    }

    hasParent(parentName) {
        let has = false;
        if (this.parents[parentName]) has = true;
        else
            for (let p of Object.values(this.parents)) {
                if (p === ed) break;
                has = p.hasParent(parentName);
                if (has) break;
            }
        return has;
    }

    hasChild(childName) {
        let has = false;
        if (this.children[childName]) has = true;
        else
            for (let p of Object.values(this.children)) {
                if (p.category === "emotion") break;
                has = p.hasChild(childName);
                if (has) break;
            }
        return has;
    }

    getRelatedDomains() {
        return this.ed.get("domains", this.label);
    }

    getRelatedLevels() {
        return this.ed.get("levels", this.label);
    }

    getRelatedProperties() {
        return this.ed.get("properties", this.label);
    }

    getRelatedValues() {
        return this.ed.get("values", this.label);
    }

    getRelatedEmotions() {
        return this.ed.get("emotions", this.label);
    }
}

class Emotion extends BaseEDModule {
    constructor(label) {
        super(label);
        this.category = "emotions";
        this.type = "emotion";
        this.setCatsAndTypes();
    }
}

class Value extends BaseEDModule {
    constructor(label, emoStr, valueRelevance, drawFunction) {
        super(label);
        this.category = "values";
        this.type = "value";
        this.valueRelevance = false;
        this.emotionLabels = this.childrenLabels = ed_utils.labelsStringToArray(emoStr.replace(/[^A-Za-z,]/g, '').toLowerCase()); //deixar só letras
        this.emotionRelevance = ed_utils.labelsStringToArray(emoStr.replace(/[^0-9,]/g, '')).map(v => { //deixar só números e vírgulas e converter em array de ints
            return parseInt(v)
        });
        this.setCatsAndTypes();
        this.draw = drawFunction ? drawFunction : ed_p5 ? ed_p5[this.label] : console.log("drawFunction not defined");
    }
}

class Property extends BaseEDModule {
    constructor(label, valuesStr, valueRelevance) {
        super(label);
        this.category = "properties";
        this.type = "property";
        this.valueRelevance = valueRelevance;
        this.valueLabels = this.childrenLabels = ed_utils.labelsStringToArray(valuesStr);
        this.setCatsAndTypes();
    }
}

class Level extends BaseEDModule {
    constructor(label, dimsStr) {
        super(label);
        this.category = "levels";
        this.type = "level";
        this.propertyLabels = this.childrenLabels = ed_utils.labelsStringToArray(dimsStr);
        this.setCatsAndTypes();
    }
}

class Domain extends BaseEDModule {
    constructor(ed, label, levelsStr) {
        super(label);
        this.category = "domains";
        this.type = "domain";
        this.ed = ed;
        this.parents = {
            ed: ed
        };
        this.levelLabels = this.childrenLabels = ed_utils.labelsStringToArray(levelsStr);
        this.setCatsAndTypes();

        this.setChildrenFromChildrenLabels();
    }
}

//ED
let ed = new class {
        constructor() {
            //EMOTIONS
            this.emotions = {
                anger: new Emotion("anger"),
                boredom: new Emotion("boredom"),
                calm: new Emotion("calm"),
                dignity: new Emotion("dignity"),
                disgust: new Emotion("disgust"),
                energy: new Emotion("energy"),
                excitement: new Emotion("excitement"),
                expectation: new Emotion("expectation"),
                fear: new Emotion("fear"),
                happiness: new Emotion("happiness"),
                masterful: new Emotion("masterful"),
                pleasantness: new Emotion("pleasantness"),
                sadness: new Emotion("sadness"),
                sleepy: new Emotion("sleepy"),
                surprise: new Emotion("surprise"),
                tenderness: new Emotion("tenderness")
            };

            //VALUES
            this.values = {
                //visual values
                //form
                line: new Value("line", "(01) Sadness, (01) Boredom"),
                point: new Value("point", ""),
                circle: new Value("circle", "(02) Happiness, (01) Pleasantness, (01) Calm"),
                oval: new Value("oval", "(01) Sleepy, (01) Sadness, (01) Pleasantness, (01) Calm, (01) Boredom"),
                triangle: new Value("triangle", "(01) Pleasantness, (01) Happiness, (01) Anger"),
                square: new Value("square", "(02) Sadness, (01) Pleasantness, (01) Excitement, (01) Anger"),
                rectangle: new Value("rectangle/polygon", ""),
                pentagon: new Value("pentagon/polygon", "(01) Sadness, (01) Anger"),
                star: new Value("star/polygon", "(01) Happiness"),
                diamond: new Value("diamond/polygon", "(01) Tenderness"),
                infinity: new Value("infinity", "(01) Tenderness, (01) Sleepy, (01) Pleasantness, (01) Happiness, (01) Excitement"),
                //regularity
                regular: new Value("regular", "(01) Sadness, (01) Pleasantness, (01) Happiness"),
                irregular: new Value("irregular", "(01) Sadness"),
                //curve
                wavy: new Value("wavy", "(01) Fear, (01) Excitement, (01) Anger"),
                straight: new Value("straight", "(01) Pleasantness, (01) Happiness, (01) Fear, (01) Calm, (01) Anger"),
                //edge
                sharp: new Value("sharp", "(02) Sadness, (02) Excitement, (02) Anger, (01) Pleasantness, (01) Fear, (01) Energy"),
                round: new Value("round", "(04) Sadness, (03) Pleasantness, (03) Happiness, (01) Calm, (01) Boredom, (01) Anger"),
                //closure
                open: new Value("open", "(01) Sadness, (01) Pleasantness, (01) Happiness, (01) Excitement"),
                close: new Value("close", "(01) Pleasantness, (01) Happiness"),
                //thickness
                thin: new Value("thin", "(01) Happiness, (01) Excitement, (01) Energy"),
                mediumThick: new Value("mediumThick", "(02) Sadness, (02) Happiness, (01) Pleasantness, (01) Anger"),
                thick: new Value("thick", "(01) Sleepy, (01) Sadness, (01) Pleasantness, (01) Calm"),
                //size
                big: new Value("big", "(02) Happiness, (02) Anger, (01) Pleasantness, (01) Energy, (01) Calm"),
                mediumSize: new Value("mediumSize", "(01) Sadness, (01) Fear"),
                small: new Value("small", "(02) Sadness, (02) Excitement, (01) Tenderness"),
                //color
                red: new Value("red", "(07) Fear, (07) Anger, (05) Happiness, (04) Masterful, (04) Excitement, (03) Sadness, (03) Energy, (02) Tenderness, (01) Surprise, (01) Boredom"),
                orange: new Value("orange", "(03) Happiness, (02) Pleasantness, (01) Tenderness, (01) Masterful, (01) Fear, (01) Excitement, (01) Energy, (01) Calm, (01) Anger"),
                yellow: new Value("yellow", "(09) Happiness, (04) Excitement, (04) Energy, (03) Sadness, (02) Tenderness, (02) Pleasantness, (02) Fear, (02) Calm, (02) Anger, (01) Sleepy, (01) Dignity, (01) Boredom"),
                green: new Value("green", "(05) Calm, (04) Happiness, (03) Tenderness, (03) Sadness, (02) Energy, (02) Boredom, (01) Pleasantness, (01) Masterful, (01) Fear, (01) Excitement, (01) Disgust, (01) Dignity"),
                blue: new Value("blue", "(06) Sadness, (05) Calm, (04) Tenderness, (04) Happiness, (04) Fear, (02) Pleasantness, (02) Dignity, (01) Disgust, (01) Boredom"),
                violet: new Value("violet", "(02) Tenderness, (02) Masterful, (02) Fear, (01) Sadness, (01) Excitement, (01) Energy, (01) Disgust"),
                purple: new Value("purple", "(04) Sadness, (03) Dignity, (02) Masterful, (01) Pleasantness, (01) Fear, (01) Disgust, (01) Calm"),
                pink: new Value("pink", "(03) Calm, (02) Happiness, (01) Pleasantness, (01) Masterful, (01) Excitement, (01) Energy"),
                brown: new Value("brown", "(02) Masterful, (02) Fear, (02) Calm, (01) Sadness, (01) Dignity, (01) Boredom, (01) Anger"),
                grey: new Value("grey/dull", "(04) Sadness, (02) Fear, (01) Tenderness, (01) Sleepy, (01) Excitement, (01) Calm, (01) Boredom"),
                black: new Value("black", "(05) Sadness, (03) Fear, (03) Anger, (01) Surprise, (01) Masterful, (01) Excitement, (01) Disgust, (01) Dignity"),
                white: new Value("white", "(02) Tenderness, (02) Calm, (01) Sleepy, (01) Sadness, (01) Happiness, (01) Fear, (01) Excitement, (01) Boredom"),
                multicolor: new Value("multicolor", "(02) Tenderness, (01) Happiness, (01) Calm"),
                bright: new Value("bright", "(05) Happiness, (03) Pleasantness, (03) Excitement, (02) Tenderness, (02) Calm, (01) Sadness, (01) Masterful, (01) Fear, (01) Anger"),
                dark: new Value("dark", "(04) Sadness, (04) Fear, (03) Anger, (02) Boredom, (01) Sleepy, (01) Dignity"),
                vivid: new Value("vivid", "(02) Happiness, (01) Sadness, (01) Pleasantness, (01) Fear, (01) Energy, (01) Dignity, (01) Boredom, (01) Anger"),
                low: new Value("low", "(01) Sadness, (01) Pleasantness, (01) Masterful, (01) Anger"),
                //transparency
                opaque: new Value("opaque",""),
                transparent: new Value("transparent",""),
                //complexity
                simple: new Value("simple", "(01) Sadness, (01) Pleasantness, (01) Happiness"),
                complex: new Value("complex ", "(01) Sadness, (01) Anger"),
                clear: new Value("clear", "(01) Pleasantness, (01) Happiness, (01) Calm"),
                crowded: new Value("crowded", "(01) Fear, (01) Anger"),
                //balance
                balanced: new Value("balanced", ""),
                imbalanced: new Value("imbalanced", ""),
                //density
                dense: new Value("dense", "(01) Sleepy, (01) Anger"),
                sparse: new Value("sparse", "(01) Fear, (01) Energy"),
                //volume
                solid: new Value("solid", "(01) Anger"),
                liquid: new Value("liquid", "(01) Fear"),
                //distribution
                structured: new Value("structured", ""),
                disordered: new Value("disordered", ""),
                //texture
                soft: new Value("soft", "(02) Happiness, (01) Sleepy, (01) Sadness, (01) Fear, (01) Calm"),
                rough: new Value("rough", "(02) Sadness, (01) Energy, (01) Anger"),
                sticky: new Value("sticky", "(01) Sleepy, (01) Fear"),
                nonSticky: new Value("nonSticky", "(01) Energy, (01) Anger"),
                //expansion
                contractive: new Value("contractive", "(01) Fear"),
                expanding: new Value("expanding", "(01) Sadness, (01) Fear, (01) Anger"),
                //movement
                fastMotion: new Value("fastMotion", "(01) Excitement, (01) Energy"),
                slowMotion: new Value("slowMotion", "(01) Sleepy, (01) Sadness"),
                staticMotion: new Value("staticMotion ", ""),
                //movementDirection
                up: new Value("up", "(03) Happiness, (01) Pleasantness, (01) Expectation, (01) Energy, (01) Anger"),
                down: new Value("down ", "(03) Sadness, (01) Sleepy, (01) Calm"),
                //symmetry
                symmetric: new Value("symmetric", "(01) Pleasantness"),
                asymmetric: new Value("asymmetric ", ""),
                //repetition
                repete: new Value("repete", "(01) Pleasantness, (01) Happiness"),
                singularity: new Value("singularity", ""),
                //rotation
                angle: new Value("angle", ""),
                //cluster
                attract: new Value("attract", ""),
                repel: new Value("repel", ""),
                //depth
                background: new Value("background", ""),
                foreground: new Value("foreground", ""),
                //diffusion
                // was removed from table
                //distance
                distance: new Value("distance",""),
                //influence
                // was removed from table
                //variation
                contrasting: new Value("contrasting", ""),
                subtle: new Value("subtle", ""),
                //overlap
                intersect: new Value("intersect", ""),
                parallelism: new Value("parallelism", ""),
                //position
                horizontal: new Value("horizontal", "(01) Pleasantness, (01) Calm"),
                vertical: new Value("vertical", ""),
                //weight
                light: new Value("light", "(02) Happiness, (01) Sadness, (01) Excitement, (01) Energy, (01) Calm"),
                heavy: new Value("heavy", "(01) Sleepy, (01) Sadness"),
                /////////////////////////////////
                //music values
                // sonance
                consonant: new Value("consonant","(05) Happiness, (04) Dignity, (03) Pleasantness, (02) Masterful, (01) Tenderness, (01) Sleepy, (01) Calm"),
                dissonant: new Value("dissonant","(05) Sadness, (05) Energy, (04) Disgust, (03) Anger, (02) Fear, (02) Excitement, (01) Masterful"),
                //harmonyIntervals
                highInterval: new Value("highInterval","(02) Energy, (01) Masterful, (01) Happiness"),
                lowInterval: new Value("lowInterval","(02) Sadness"),
                //melodyIntervals
                m2: new Value("m2","(01) Sadness, (01) Energy"),
                m7: new Value("m7","(01) Pleasantness, (01) Happiness"),
                M3: new Value("M3","(01) Masterful, (01) Happiness"),
                M6: new Value("M6","(01) Happiness"),
                P4: new Value("P4","(01) Pleasantness, (01) Happiness"),
                P5: new Value("P5","(01) Pleasantness, (01) Happiness, (01) Energy"),
                Ag4: new Value("Ag4","(01) Anger"),
                unison: new Value("unison","(01) Masterful"),
                octave: new Value("octave","(02) Masterful, (01) Pleasantness, (01) Happiness, (01) Energy"),
                tritones: new Value("tritones","(01) Energy"),
                //melodyPitchRange
                widePitchRange: new Value("widePitchRange","(03) Fear, (02) Happiness, (01) Masterful"),
                narrowPitchRange: new Value("narrowPitchRange","(02) Sadness, (01) Tenderness, (01) Pleasantness, (01) Masterful, (01) Dignity, (01) Calm"),
                //pitch
                highPitch: new Value("highPitch","(06) Happiness, (02) Masterful, (02) Energy, (01) Tenderness, (01) Surprise, (01) Sleepy, (01) Fear, (01) Excitement, (01) Dignity, (01) Calm, (01) Anger"),
                lowPitch: new Value("lowPitch","(09) Sadness, (04) Dignity, (02) Energy, (01) Pleasantness, (01) Masterful, (01) Fear, (01) Excitement, (01) Calm, (01) Boredom"),
                //direction
                ascending: new Value("ascending","(07) Happiness, (02) Fear, (02) Anger, (01) Surprise, (01) Sadness, (01) Masterful, (01) Dignity, (01) Calm"),
                descending: new Value("descending","(03) Sadness, (01) Tenderness, (01) Pleasantness, (01) Masterful, (01) Happiness, (01) Excitement, (01) Dignity, (01) Boredom"),
                // motion
                stepwise: new Value("stepwise","(01) Boredom"),
                intervallicLeaps: new Value("intervallicLeaps","(01) Excitement, (01) Energy"),
                stepwiseLeaps: new Value("stepwiseLeaps","(01) Calm"),
                // pitchVariation
                largePitchVariation: new Value("largePitchVariation","(04) Happiness, (02) Fear, (01) Surprise, (01) Pleasantness, (01) Energy, (01) Anger"),
                mediumPitchVariation: new Value("mediumPitchVariation","(01) Anger"),
                smallPitchVariation: new Value("smallPitchVariation","(03) Sadness, (02) Fear, (01) Tenderness, (01) Happiness, (01) Disgust, (01) Boredom, (01) Anger"),
                // mode
                major: new Value("major","(13) Happiness, (02) Dignity, (02) Calm, (01) Tenderness"),
                minor: new Value("minor","(13) Sadness, (02) Fear, (01) Sleepy, (01) Disgust, (01) Dignity, (01) Anger"),
                // tonality
                atonal: new Value("atonal","(01) Anger"),
                tonal: new Value("tonal","(01) Pleasantness, (01) Happiness, (01) Calm, (01) Boredom"),
                // amplitude
                roundAmplitude: new Value("roundAmplitude","(04) Sadness, (02) Tenderness, (02) Fear, (01) Masterful, (01) Disgust, (01) Boredom"),
                sharpAmplitude: new Value("sharpAmplitude","(04) Happiness, (03) Anger, (02) Energy, (01) Surprise, (01) Pleasantness, (01) Fear, (01) Dignity"),
                // articulation
                staccato: new Value("staccato","(11) Anger, (02) Energy"),
                legato: new Value("legato","(14) Sadness, (09) Tenderness, (04) Anger, (02) Happiness, (02) Dignity, (01) Fear, (01) Expectation, (01) Calm"),
                // articulationVariation
                largeArticulationVariation: new Value("largeArticulationVariation","(07) Happiness, (05) Fear"),
                mediumArticulationVariation: new Value("mediumArticulationVariation","(06) Anger"),
                smallArticulationVariation: new Value("smallArticulationVariation","(05) Sadness, (01) Tenderness"),
                // loudness
                loud: new Value("loud","(25) Anger, (16) Happiness, (04) Energy, (02) Masterful, (01) Excitement, (01) Dignity"),
                moderate: new Value("moderate","(14) Happiness, (02) Sadness, (02) Fear, (02) Dignity"),
                softLoudness: new Value("softLoudness","(24) Sadness, (14) Tenderness, (12) Fear, (01) Pleasantness, (01) Happiness, (01) Dignity, (01) Calm"),
                // loudnessVariation
                largeLoudnessVaration: new Value("largeLoudnessVaration","(06) Fear, (03) Happiness, (03) Anger, (01) Sadness"),
                moderateLoudnessVariation: new Value("moderateLoudnessVariation","(01) Sadness, (01) Happiness"),
                smallLoudnessVariation: new Value("smallLoudnessVariation","(06) Happiness, (03) Tenderness, (03) Sadness, (03) Anger, (01) Pleasantness, (01) Energy"),
                rapidChanges: new Value("rapidChanges","(01) Pleasantness, (01) Fear"),
                fewChanges: new Value("fewChanges","(01) Sadness, (01) Happiness, (01) Dignity, (01) Calm"),
                // vibrato
                fastVibrato: new Value("fastVibrato","(01) Fear"),
                slowVibrato: new Value("slowVibrato","(02) Sadness"),
                irregularVibrato: new Value("irregularVibrato","(02) Fear"),
                largeExtent: new Value("largeExtent","(08) Anger, (02) Happiness, (02) Fear, (02) Energy, (01) Sadness"),
                smallExtent: new Value("smallExtent","(03) Sadness, (03) Fear, (02) Happiness, (01) Tenderness"),
                // timbre
                manyHarmonics: new Value("manyHarmonics","(05) Anger, (02) Happiness, (01) Masterful, (01) Fear, (01) Energy, (01) Disgust"),
                fewHarmonics: new Value("fewHarmonics","(04) Sadness, (02) Tenderness, (02) Fear, (01) Pleasantness, (01) Happiness, (01) Boredom"),
                sharpTimbre: new Value("sharp","(05) Anger"),
                softTimbre: new Value("soft","(02) Tenderness, (01) Sadness"),
                //highFreqEnergy
                highFrequency: new Value("highFrequency","(10) Anger, (02) Fear, (01) Happiness"),
                mediumFrequency: new Value("mediumFrequency","(09) Happiness"),
                lowFrequency: new Value("lowFrequency","(08) Sadness, (06) Tenderness, (05) Fear, (01) Happiness"),
                //attack
                fastAttack: new Value("fastAttack","(09) Anger, (05) Happiness, (01) Fear"),
                slowAttack: new Value("slowAttack","(08) Sadness, (03) Tenderness, (02) Fear, (01) Happiness"),
                //pause
                beforeClosure: new Value("beforeClosure",""),
                afterClosure: new Value("afterClosure",""),
                // rhythm
                regularRhythm: new Value("regularRhythm","(02) Happiness, (02) Dignity, (01) Pleasantness, (01) Masterful, (01) Calm"),
                irregularRhythm: new Value("irregularRhythm","(01) Pleasantness"),
                complexRhythm: new Value("complexRhythm","(02) Anger"),
                variedRhythm: new Value("variedRhythm","(01) Happiness"),
                firmRhythm: new Value("firmRhythm","(02) Sadness, (01) Masterful, (01) Excitement, (01) Dignity"),
                flowingRhythm: new Value("flowingRhythm","(02) Happiness, (01) Sleepy, (01) Dignity, (01) Calm"),
                //tempo
                fast: new Value("fast","(43) Happiness, (24) Anger, (10) Fear, (08) Energy, (06) Pleasantness, (05) Surprise, (02) Masterful, (01) Expectation, (01) Dignity"),
                moderate: new Value("moderate","(04) Happiness, (03) Fear, (01) Tenderness, (01) Anger"),
                slow: new Value("slow","(40) Sadness, (14) Tenderness, (08) Dignity, (04) Fear, (03) Calm, (01) Sleepy, (01) Excitement, (01) Disgust, (01) Boredom"),
                //tempoVariation
                largeTempoVariation: new Value("largeTempoVariation","(09) Fear, (03) Tenderness, (03) Sadness"),
                mediumTempoVariation: new Value("mediumTempoVariation","(03) Sadness, (03) Anger, (01) Happiness"),
                lowTempoVariation: new Value("lowTempoVariation","(07) Happiness, (02) Anger, (01) Tenderness, (01) Sadness"),
                //noteDensity
                highDensity: new Value("highDensity","(01) Happiness, (01) Fear"),
                lowDensity: new Value("lowDensity","(01) Sadness"),
                //pattern
                repetition: new Value("repetition","(01) Expectation"),
                sequence: new Value("sequence",""),
                //noteDurationContrast
                sharpBig: new Value("sharpBig","(05) Anger, (03) Happiness, (02) Fear"),
                softSmall: new Value("softSmall","(05) Tenderness, (04) Sadness, (02) Happiness")
            };

            //PROPERTIES
            this.properties = {
                // visual properties
                form: new Property("form", "line,point,circle,oval,triangle,square,rectangle,pentagon,star,diamond,infinity", true),
                regularity: new Property("regularity", "regular,irregular"),
                curve: new Property("curve", "wavy,straight"),
                edge: new Property("edge", "sharp,round", true),
                closure: new Property("closure", "open,close"),
                thickness: new Property("thickness", "thin,mediumThick,thick", true),
                size: new Property("size", "big,mediumSize,small", true),
                hue: new Property("hue", "red,orange,yellow,green,blue,violet,purple,pink,brown,grey,black,white,multicolor", true),
                brightness: new Property("brightness", "bright,dark", true),
                saturation: new Property("saturation", "vivid,low", true),
                transparency: new Property("transparency", ""),
                complexity: new Property("complexity", "simple,complex,clear,crowded", true),
                balance: new Property("balance", "balanced,imbalanced"),
                density: new Property("density", "dense,sparse"),
                volume: new Property("volume", "solid,liquid"),
                distribution: new Property("distribution", "structured,disordered"),
                texture: new Property("texture", "soft,rough,sticky,nonSticky", true),
                expansion: new Property("expansion", "contractive,expanding"),
                movement: new Property("movement", "fastMotion,slowMotion,staticMotion"),
                movementDirection: new Property("movementDirection", "up,down", true),
                symmetry: new Property("symmetry", "symmetric,asymmetric"),
                repetition: new Property("repetition", "repete,singularity"),
                rotation: new Property("rotation", "angle"),
                cluster: new Property("cluster", "attract,repel"),
                depth: new Property("depth", "background,foreground"),
                //diffusion: new Property("diffusion", ""),
                distance: new Property("distance", ""),
                //influence: new Property("influence", ""),
                variation: new Property("variation", "contrasting,subtle"),
                overlap: new Property("overlap", "intersect,parallelism"),
                position: new Property("position", "horizontal,vertical"),
                weight: new Property("weight", "light,heavy"),
                /////////////////////////////////
                // music properties
                harmonySonance: new Property("harmonySonance","consonant,dissonant", true),
                harmonyIntervals: new Property("harmonyIntervals","highInterval,lowInterval"),
                melodyIntervals: new Property("melodyIntervals","m2,m7,M3,M6,P4,P5,Ag4,unison,octave,tritones"),
                melodyPitchRange: new Property("melodyPitchRange","widePitchRange,narrowPitchRange"),
                pitch: new Property("pitch","highPitch,lowPitch", true),
                direction: new Property("direction","ascending,descending"),
                motion: new Property("motion","stepwise,intervallicLeaps,septwiseLeaps"),
                pitchVariation: new Property("pitchVariation","largePitchVariation,mediumPitchVariation,smallPitchVariation"),
                mode: new Property("mode","major,minor", true),
                tonality: new Property("tonality","atonal,tonal"),
                amplitude: new Property("amplitude","roundAmplitude,sharpAmplitude"),
                articulation: new Property("articulation","staccato,legato", true),
                articulationVariation: new Property("articulationVariation","largeArticulationVariation,mediumArticulationVariation,smallArticulationVariation"),
                loudness: new Property("loudness","loud,moderate,softLoudness", true),
                loudnessVariation: new Property("loudnessVariation","largeLoudnessVaration,moderateLoudnessVariation,smallLoudnessVariation,rapidChanges,fewChanges"),
                vibrato: new Property("vibrato","fastVibrato,slowVibrato,irregularVibrato,largeExtent,smallExtent", true),
                timbre: new Property("timbre","manyHarmonics,fewHarmonics,sharpTimbre,softTimbre", true),
                highFrequencyEnergy: new Property("highFrequencyEnergy","highFrequency,mediumFrequency,lowFrequency"),
                attack: new Property("attack","fastAttack,slowAttack", true),
                pause: new Property("pause","afterClosure,beforeClosure"),
                rhythm: new Property("rhythm","regularRhythm,irregularRhythm,complexRhythm,variedRhythm,firmRhythm,flowingRhythm"),
                tempo: new Property("tempo","fast,moderate,slow", true),
                tempoVariation: new Property("tempoVariation","largeTempoVariation,mediumTempoVariation,lowTempoVariation"),
                noteDensity: new Property("noteDensity","highDensity,lowDensity"),
                pattern: new Property("pattern","repetition,sequence"),
                noteDurationContrast: new Property("noteDurationContrast","sharpBig,softSmall")
            };

            //LEVELS
            this.levels = {
                //image
                low: new Level("low", "form,regularity,curve,edge,closure,thickness,size,hue,saturation,brightness,transparency"),
                high: new Level("high", "balance,complexity,density,distribution,position,texture,volume"),
                manipulations: new Level("manipulations","expansion,movement,movementDirection,symmetry,repetition,rotation"),
                relations: new Level("relations","cluster,depth,distance,variation,overlap,weight"),
                /////////////////////////////////
                //music
                harmonyMelody: new Level("harmonyMelody","harmonySonance,harmonyIntervals,melodyIntervals,melodyPitchRange,pitch,direction,motion,pitchVariation,mode,tonality"),
                dynamics: new Level("dynamics","amplitude,articulation,articulationVariation,loudness,loudnessVariation,vibrato"),
                texture: new Level("texture","timbre,highFrequencyEnergy,attack"),
                temporalOrganization: new Level("temporalOrganization","pause,rhythm,tempo,tempoVariation,noteDensity,pattern,noteDurationContrast")
            };

            //DOMAINS
            this.domains = {
                image: new Domain(this, "image", "high,low,manipulations,relations"),
                music: new Domain(this, "music", "harmonyMelody,dynamics,texture,temporalOrganization")
            };
        }

        get(category, filterName) { //p.e. "emotions" of "highLevel"
            let emos = {};
            if (!filterName) emos = this[category];
            else
                for (let v of Object.values(this[category]))
                    if (v.hasParent(filterName)) emos[v.label] = v;
                    else if (v.hasChild(filterName)) emos[v.label] = v;
            return emos;
        }

        getEmotion(emotionName) {
            return this.emotions[emotionName];
        }

        getValue(propertyName) {
            return this.properties[propertyName];
        }

        getProperty(propertyName) {
            return this.properties[propertyName];
        }

        getLevel(levelName) {
            return this.levels[levelName];
        }

        getDomain(domainName) {
            return this.domains[domainName];
        }

        getCategory(categoryName) {
            return this[categoryName];
        }

        getEmotions(filterName) {
            return filterName ? this.get("emotions", filterName) : this.emotions;
        }

        getValues(filterName) {
            return filterName ? this.get("properties", filterName) : this.properties;
        }

        getProperties(filterName) {
            return filterName ? this.get("properties", filterName) : this.properties;
        }

        getLevels(filterName) {
            return filterName ? this.get("levels", filterName) : this.levels;
        }

        getDomains(filterName) {
            return filterName ? this.get("domains", filterName) : this.domains;
        }

    },
    emotionsDecipher = ed;
/*

    let ed_utils = {
        categoriesHierarchy: ["ed", "domains", "levels", "properties", "values", "emotions"],
        typesHierarchy: ["ed", "domain", "level", "property", "value", "emotion"],
        getFromCategory: (category, labels) => {
            let r = {};
            if (typeof labels === "string") labels = this.labelsStringToArray(labels);
            for (let l of labels)
                if (category[l]) r[l] = category[l];
            return r;
        },
        labelsStringToArray: labelsStr => {
            if (typeof labelsStr === "string") labelsStr = labelsStr.replace(/\s/g, '').split(",");
            return labelsStr === "" ? [] : labelsStr
        },
    }
    
    class BaseEDModule {
        constructor(label) {
            this.allLabels = label.replace(/\s/g, '').split("/");
            this.label = this.allLabels[0];
            this.otherlabels = this.allLabels.slice(1);
            this.category = "";
            this.type = "";
            this.parents = {};
            this.children = {};
            this.parentsCategory = "";
            this.parentsType = "";
            this.childrenCategory = "";
            this.childrenType = ""
            this.ed = null;
            this.childrenLabels = [];
        }
    
        randomChild() {
            let childrenArr = Object.values(this.children);
            let index = Math.floor(Math.random() * childrenArr.length);
            return childrenArr[index];
        }
    
        randomParent() {
            let parentsArr = Object.values(this.parents);
            let index = Math.floor(Math.random() * parentsArr.length);
            return parentsArr[index];
        }
    
        setCatsAndTypes() {
            let hindex = ed_utils.categoriesHierarchy.indexOf(this.category);
            this.parentsCategory = ed_utils.categoriesHierarchy[hindex - 1];
            this.parentsType = ed_utils.typesHierarchy[hindex - 1];
            this.childrenCategory = ed_utils.categoriesHierarchy[hindex + 1];
            this.childrenType = ed_utils.typesHierarchy[hindex + 1];
        }
    
        setChildrenFromChildrenLabels() {
            this[this.childrenCategory] = ed_utils.getFromCategory(this.ed[this.childrenCategory], this.childrenLabels);
    
            let chs = this[this.childrenCategory];
            for (let k of Object.keys(chs)) {
                this.children[k] = chs[k];
                chs[k].setParent(this);
                chs[k].setChildrenFromChildrenLabels();
            }
            return this.children;
        }
    
        setParent(parent) {
            this.parents[parent.label] = parent;
            if (!this.ed) this.ed = parent.ed;
        }
    
        hasParent(parentName) {
            let has = false;
            if (this.parents[parentName]) has = true;
            else
                for (let p of Object.values(this.parents)) {
                    if (p === ed) break;
                    has = p.hasParent(parentName);
                    if (has) break;
                }
            return has;
        }
    
        hasChild(childName) {
            let has = false;
            if (this.children[childName]) has = true;
            else
                for (let p of Object.values(this.children)) {
                    if (p.category === "emotion") break;
                    has = p.hasChild(childName);
                    if (has) break;
                }
            return has;
        }
    
        getRelatedDomains() {
            return this.ed.get("domains", this.label);
        }
    
        getRelatedLevels() {
            return this.ed.get("levels", this.label);
        }
    
        getRelatedProperties() {
            return this.ed.get("properties", this.label);
        }
    
        getRelatedValues() {
            return this.ed.get("values", this.label);
        }
    
        getRelatedEmotions() {
            return this.ed.get("emotions", this.label);
        }
    }
    
    class Emotion extends BaseEDModule {
        constructor(label) {
            super(label);
            this.category = "emotions";
            this.type = "emotion";
            this.setCatsAndTypes();
        }
    }
    
    class Value extends BaseEDModule {
        constructor(label, emoStr, valueRelevance, drawFunction) {
            super(label);
            this.category = "values";
            this.type = "value";
            this.valueRelevance = false;
            this.emotionLabels = this.childrenLabels = ed_utils.labelsStringToArray(emoStr.replace(/[^A-Za-z,]/g, '').toLowerCase()); //deixar só letras
            this.emotionRelevance = ed_utils.labelsStringToArray(emoStr.replace(/[^0-9,]/g, '')).map(v => { //deixar só números e vírgulas e converter em array de ints
                return parseInt(v)
            });
            this.setCatsAndTypes();
            this.draw = drawFunction ? drawFunction : ed_p5 ? ed_p5[this.label] : console.log("drawFunction not defined");
        }
    }
    
    class Property extends BaseEDModule {
        constructor(label, valuesStr) {
            super(label);
            this.category = "properties";
            this.type = "property";
            this.valueLabels = this.childrenLabels = ed_utils.labelsStringToArray(valuesStr);
            this.setCatsAndTypes();
        }
    }
    
    class Level extends BaseEDModule {
        constructor(label, dimsStr) {
            super(label);
            this.category = "levels";
            this.type = "level";
            this.propertyLabels = this.childrenLabels = ed_utils.labelsStringToArray(dimsStr);
            this.setCatsAndTypes();
        }
    }
    
    class Domain extends BaseEDModule {
        constructor(ed, label, levelsStr) {
            super(label);
            this.category = "domains";
            this.type = "domain";
            this.ed = ed;
            this.parents = {
                ed: ed
            };
            this.levelLabels = this.childrenLabels = ed_utils.labelsStringToArray(levelsStr);
            this.setCatsAndTypes();
    
            this.setChildrenFromChildrenLabels();
        }
    }
    
    //ED
    let ed = new class {
            constructor() {
                //EMOTIONS
                this.emotions = {
                    anger: new Emotion("anger"),
                    boredom: new Emotion("boredom"),
                    calm: new Emotion("calm"),
                    dignity: new Emotion("dignity"),
                    disgust: new Emotion("disgust"),
                    energy: new Emotion("energy"),
                    excitement: new Emotion("excitement"),
                    expectation: new Emotion("expectation"),
                    fear: new Emotion("fear"),
                    happiness: new Emotion("happiness"),
                    masterful: new Emotion("masterful"),
                    pleasantness: new Emotion("pleasantness"),
                    sadness: new Emotion("sadness"),
                    sleepy: new Emotion("sleepy"),
                    surprise: new Emotion("surprise"),
                    tenderness: new Emotion("tenderness")
                };
    
                //VALUES
                this.values = {
                    //visual values
                    //form
                    line: new Value("line", "(01) Sadness, (01) Boredom"),
                    point: new Value("point", ""),
                    circle: new Value("circle", "(02) Happiness, (01) Pleasantness, (01) Calm"),
                    oval: new Value("oval", "(01) Sleepy, (01) Sadness, (01) Pleasantness, (01) Calm, (01) Boredom"),
                    triangle: new Value("triangle", "(01) Pleasantness, (01) Happiness, (01) Anger"),
                    square: new Value("square", "(02) Sadness, (01) Pleasantness, (01) Excitement, (01) Anger"),
                    rectangle: new Value("rectangle/polygon", ""),
                    pentagon: new Value("pentagon/polygon", "(01) Sadness, (01) Anger"),
                    star: new Value("star/polygon", "(01) Happiness"),
                    diamond: new Value("diamond/polygon", "(01) Tenderness"),
                    infinity: new Value("infinity", "(01) Tenderness, (01) Sleepy, (01) Pleasantness, (01) Happiness, (01) Excitement"),
                    //regularity
                    regular: new Value("regular", "(01) Sadness, (01) Pleasantness, (01) Happiness"),
                    irregular: new Value("irregular", "(01) Sadness"),
                    //curve
                    wavy: new Value("wavy", "(01) Fear, (01) Excitement, (01) Anger"),
                    straight: new Value("straight", "(01) Pleasantness, (01) Happiness, (01) Fear, (01) Calm, (01) Anger"),
                    //edge
                    sharp: new Value("sharp", "(02) Sadness, (02) Excitement, (02) Anger, (01) Pleasantness, (01) Fear, (01) Energy"),
                    round: new Value("round", "(04) Sadness, (03) Pleasantness, (03) Happiness, (01) Calm, (01) Boredom, (01) Anger"),
                    //closure
                    open: new Value("open", "(01) Sadness, (01) Pleasantness, (01) Happiness, (01) Excitement"),
                    close: new Value("close", "(01) Pleasantness, (01) Happiness"),
                    //thickness
                    thin: new Value("thin", "(01) Happiness, (01) Excitement, (01) Energy"),
                    mediumThick: new Value("mediumThick", "(02) Sadness, (02) Happiness, (01) Pleasantness, (01) Anger"),
                    thick: new Value("thick", "(01) Sleepy, (01) Sadness, (01) Pleasantness, (01) Calm"),
                    //size
                    big: new Value("big", "(02) Happiness, (02) Anger, (01) Pleasantness, (01) Energy, (01) Calm"),
                    mediumSize: new Value("mediumSize", "(01) Sadness, (01) Fear"),
                    small: new Value("small", "(02) Sadness, (02) Excitement, (01) Tenderness"),
                    //color
                    red: new Value("red", "(07) Fear, (07) Anger, (05) Happiness, (04) Masterful, (04) Excitement, (03) Sadness, (03) Energy, (02) Tenderness, (01) Surprise, (01) Boredom"),
                    orange: new Value("orange", "(03) Happiness, (02) Pleasantness, (01) Tenderness, (01) Masterful, (01) Fear, (01) Excitement, (01) Energy, (01) Calm, (01) Anger"),
                    yellow: new Value("yellow", "(09) Happiness, (04) Excitement, (04) Energy, (03) Sadness, (02) Tenderness, (02) Pleasantness, (02) Fear, (02) Calm, (02) Anger, (01) Sleepy, (01) Dignity, (01) Boredom"),
                    green: new Value("green", "(05) Calm, (04) Happiness, (03) Tenderness, (03) Sadness, (02) Energy, (02) Boredom, (01) Pleasantness, (01) Masterful, (01) Fear, (01) Excitement, (01) Disgust, (01) Dignity"),
                    blue: new Value("blue", "(06) Sadness, (05) Calm, (04) Tenderness, (04) Happiness, (04) Fear, (02) Pleasantness, (02) Dignity, (01) Disgust, (01) Boredom"),
                    violet: new Value("violet", "(02) Tenderness, (02) Masterful, (02) Fear, (01) Sadness, (01) Excitement, (01) Energy, (01) Disgust"),
                    purple: new Value("purple", "(04) Sadness, (03) Dignity, (02) Masterful, (01) Pleasantness, (01) Fear, (01) Disgust, (01) Calm"),
                    pink: new Value("pink", "(03) Calm, (02) Happiness, (01) Pleasantness, (01) Masterful, (01) Excitement, (01) Energy"),
                    brown: new Value("brown", "(02) Masterful, (02) Fear, (02) Calm, (01) Sadness, (01) Dignity, (01) Boredom, (01) Anger"),
                    grey: new Value("grey/dull", "(04) Sadness, (02) Fear, (01) Tenderness, (01) Sleepy, (01) Excitement, (01) Calm, (01) Boredom"),
                    black: new Value("black", "(05) Sadness, (03) Fear, (03) Anger, (01) Surprise, (01) Masterful, (01) Excitement, (01) Disgust, (01) Dignity"),
                    white: new Value("white", "(02) Tenderness, (02) Calm, (01) Sleepy, (01) Sadness, (01) Happiness, (01) Fear, (01) Excitement, (01) Boredom"),
                    multicolor: new Value("multicolor", "(02) Tenderness, (01) Happiness, (01) Calm"),
                    bright: new Value("bright", "(05) Happiness, (03) Pleasantness, (03) Excitement, (02) Tenderness, (02) Calm, (01) Sadness, (01) Masterful, (01) Fear, (01) Anger"),
                    dark: new Value("dark", "(04) Sadness, (04) Fear, (03) Anger, (02) Boredom, (01) Sleepy, (01) Dignity"),
                    vivid: new Value("vivid", "(02) Happiness, (01) Sadness, (01) Pleasantness, (01) Fear, (01) Energy, (01) Dignity, (01) Boredom, (01) Anger"),
                    low: new Value("low", "(01) Sadness, (01) Pleasantness, (01) Masterful, (01) Anger"),
                    //transparency
                    opaque: new Value("opaque",""),
                    transparent: new Value("transparent",""),
                    //complexity
                    simple: new Value("simple", "(01) Sadness, (01) Pleasantness, (01) Happiness"),
                    complex: new Value("complex ", "(01) Sadness, (01) Anger"),
                    clear: new Value("clear", "(01) Pleasantness, (01) Happiness, (01) Calm"),
                    crowded: new Value("crowded", "(01) Fear, (01) Anger"),
                    //balance
                    balanced: new Value("balanced", ""),
                    imbalanced: new Value("imbalanced", ""),
                    //density
                    dense: new Value("dense", "(01) Sleepy, (01) Anger"),
                    sparse: new Value("sparse", "(01) Fear, (01) Energy"),
                    //volume
                    solid: new Value("solid", "(01) Anger"),
                    liquid: new Value("liquid", "(01) Fear"),
                    //distribution
                    structured: new Value("structured", ""),
                    disordered: new Value("disordered", ""),
                    //texture
                    soft: new Value("soft", "(02) Happiness, (01) Sleepy, (01) Sadness, (01) Fear, (01) Calm"),
                    rough: new Value("rough", "(02) Sadness, (01) Energy, (01) Anger"),
                    sticky: new Value("sticky", "(01) Sleepy, (01) Fear"),
                    nonSticky: new Value("nonSticky", "(01) Energy, (01) Anger"),
                    //expansion
                    contractive: new Value("contractive", "(01) Fear"),
                    expanding: new Value("expanding", "(01) Sadness, (01) Fear, (01) Anger"),
                    //movement
                    fastMotion: new Value("fastMotion", "(01) Excitement, (01) Energy"),
                    slowMotion: new Value("slowMotion", "(01) Sleepy, (01) Sadness"),
                    staticMotion: new Value("staticMotion ", ""),
                    //movementDirection
                    up: new Value("up", "(03) Happiness, (01) Pleasantness, (01) Expectation, (01) Energy, (01) Anger"),
                    down: new Value("down ", "(03) Sadness, (01) Sleepy, (01) Calm"),
                    //symmetry
                    symmetric: new Value("symmetric", "(01) Pleasantness"),
                    asymmetric: new Value("asymmetric ", ""),
                    //repetition
                    repete: new Value("repete", "(01) Pleasantness, (01) Happiness"),
                    singularity: new Value("singularity", ""),
                    //rotation
                    angle: new Value("angle", ""),
                    //cluster
                    attract: new Value("attract", ""),
                    repel: new Value("repel", ""),
                    //depth
                    background: new Value("background", ""),
                    foreground: new Value("foreground", ""),
                    //diffusion
                    // was removed from table
                    //distance
                    distance: new Value("distance",""),
                    //influence
                    // was removed from table
                    //variation
                    contrasting: new Value("contrasting", ""),
                    subtle: new Value("subtle", ""),
                    //overlap
                    intersect: new Value("intersect", ""),
                    parallelism: new Value("parallelism", ""),
                    //position
                    horizontal: new Value("horizontal", "(01) Pleasantness, (01) Calm"),
                    vertical: new Value("vertical", ""),
                    //weight
                    light: new Value("light", "(02) Happiness, (01) Sadness, (01) Excitement, (01) Energy, (01) Calm"),
                    heavy: new Value("heavy", "(01) Sleepy, (01) Sadness"),
                    /////////////////////////////////
                    //music values
                    // sonance
                    consonant: new Value("consonant","(05) Happiness, (04) Dignity, (03) Pleasantness, (02) Masterful, (01) Tenderness, (01) Sleepy, (01) Calm"),
                    dissonant: new Value("dissonant","(05) Sadness, (05) Energy, (04) Disgust, (03) Anger, (02) Fear, (02) Excitement, (01) Masterful"),
                    //harmonyIntervals
                    highInterval: new Value("highInterval","(02) Energy, (01) Masterful, (01) Happiness"),
                    lowInterval: new Value("lowInterval","(02) Sadness"),
                    //melodyIntervals
                    m2: new Value("m2","(01) Sadness, (01) Energy"),
                    m7: new Value("m7","(01) Pleasantness, (01) Happiness"),
                    M3: new Value("M3","(01) Masterful, (01) Happiness"),
                    M6: new Value("M6","(01) Happiness"),
                    P4: new Value("P4","(01) Pleasantness, (01) Happiness"),
                    P5: new Value("P5","(01) Pleasantness, (01) Happiness, (01) Energy"),
                    Ag4: new Value("Ag4","(01) Anger"),
                    unison: new Value("unison","(01) Masterful"),
                    octave: new Value("octave","(02) Masterful, (01) Pleasantness, (01) Happiness, (01) Energy"),
                    tritones: new Value("tritones","(01) Energy"),
                    //melodyPitchRange
                    widePitchRange: new Value("widePitchRange","(03) Fear, (02) Happiness, (01) Masterful"),
                    narrowPitchRange: new Value("narrowPitchRange","(02) Sadness, (01) Tenderness, (01) Pleasantness, (01) Masterful, (01) Dignity, (01) Calm"),
                    //pitch
                    highPitch: new Value("highPitch","(06) Happiness, (02) Masterful, (02) Energy, (01) Tenderness, (01) Surprise, (01) Sleepy, (01) Fear, (01) Excitement, (01) Dignity, (01) Calm, (01) Anger"),
                    lowPitch: new Value("lowPitch","(09) Sadness, (04) Dignity, (02) Energy, (01) Pleasantness, (01) Masterful, (01) Fear, (01) Excitement, (01) Calm, (01) Boredom"),
                    //direction
                    ascending: new Value("ascending","(07) Happiness, (02) Fear, (02) Anger, (01) Surprise, (01) Sadness, (01) Masterful, (01) Dignity, (01) Calm"),
                    descending: new Value("descending","(03) Sadness, (01) Tenderness, (01) Pleasantness, (01) Masterful, (01) Happiness, (01) Excitement, (01) Dignity, (01) Boredom"),
                    // motion
                    stepwise: new Value("stepwise","(01) Boredom"),
                    intervallicLeaps: new Value("intervallicLeaps","(01) Excitement, (01) Energy"),
                    stepwiseLeaps: new Value("stepwiseLeaps","(01) Calm"),
                    // pitchVariation
                    largePitchVariation: new Value("largePitchVariation","(04) Happiness, (02) Fear, (01) Surprise, (01) Pleasantness, (01) Energy, (01) Anger"),
                    mediumPitchVariation: new Value("mediumPitchVariation","(01) Anger"),
                    smallPitchVariation: new Value("smallPitchVariation","(03) Sadness, (02) Fear, (01) Tenderness, (01) Happiness, (01) Disgust, (01) Boredom, (01) Anger"),
                    // mode
                    major: new Value("major","(13) Happiness, (02) Dignity, (02) Calm, (01) Tenderness"),
                    minor: new Value("minor","(13) Sadness, (02) Fear, (01) Sleepy, (01) Disgust, (01) Dignity, (01) Anger"),
                    // tonality
                    atonal: new Value("atonal","(01) Anger"),
                    tonal: new Value("tonal","(01) Pleasantness, (01) Happiness, (01) Calm, (01) Boredom"),
                    // amplitude
                    roundAmplitude: new Value("roundAmplitude","(04) Sadness, (02) Tenderness, (02) Fear, (01) Masterful, (01) Disgust, (01) Boredom"),
                    sharpAmplitude: new Value("sharpAmplitude","(04) Happiness, (03) Anger, (02) Energy, (01) Surprise, (01) Pleasantness, (01) Fear, (01) Dignity"),
                    // articulation
                    staccato: new Value("staccato","(11) Anger, (02) Energy"),
                    legato: new Value("legato","(14) Sadness, (09) Tenderness, (04) Anger, (02) Happiness, (02) Dignity, (01) Fear, (01) Expectation, (01) Calm"),
                    // articulationVariation
                    largeArticulationVariation: new Value("largeArticulationVariation","(07) Happiness, (05) Fear"),
                    mediumArticulationVariation: new Value("mediumArticulationVariation","(06) Anger"),
                    smallArticulationVariation: new Value("smallArticulationVariation","(05) Sadness, (01) Tenderness"),
                    // loudness
                    loud: new Value("loud","(25) Anger, (16) Happiness, (04) Energy, (02) Masterful, (01) Excitement, (01) Dignity"),
                    moderate: new Value("moderate","(14) Happiness, (02) Sadness, (02) Fear, (02) Dignity"),
                    softLoudness: new Value("softLoudness","(24) Sadness, (14) Tenderness, (12) Fear, (01) Pleasantness, (01) Happiness, (01) Dignity, (01) Calm"),
                    // loudnessVariation
                    largeLoudnessVaration: new Value("largeLoudnessVaration","(06) Fear, (03) Happiness, (03) Anger, (01) Sadness"),
                    moderateLoudnessVariation: new Value("moderateLoudnessVariation","(01) Sadness, (01) Happiness"),
                    smallLoudnessVariation: new Value("smallLoudnessVariation","(06) Happiness, (03) Tenderness, (03) Sadness, (03) Anger, (01) Pleasantness, (01) Energy"),
                    rapidChanges: new Value("rapidChanges","(01) Pleasantness, (01) Fear"),
                    fewChanges: new Value("fewChanges","(01) Sadness, (01) Happiness, (01) Dignity, (01) Calm"),
                    // vibrato
                    fastVibrato: new Value("fastVibrato","(01) Fear"),
                    slowVibrato: new Value("slowVibrato","(02) Sadness"),
                    irregularVibrato: new Value("irregularVibrato","(02) Fear"),
                    largeExtent: new Value("largeExtent","(08) Anger, (02) Happiness, (02) Fear, (02) Energy, (01) Sadness"),
                    smallExtent: new Value("smallExtent","(03) Sadness, (03) Fear, (02) Happiness, (01) Tenderness"),
                    // timbre
                    manyHarmonics: new Value("manyHarmonics","(05) Anger, (02) Happiness, (01) Masterful, (01) Fear, (01) Energy, (01) Disgust"),
                    fewHarmonics: new Value("fewHarmonics","(04) Sadness, (02) Tenderness, (02) Fear, (01) Pleasantness, (01) Happiness, (01) Boredom"),
                    sharpTimbre: new Value("sharp","(05) Anger"),
                    softTimbre: new Value("soft","(02) Tenderness, (01) Sadness"),
                    //highFreqEnergy
                    highFrequency: new Value("highFrequency","(10) Anger, (02) Fear, (01) Happiness"),
                    mediumFrequency: new Value("mediumFrequency","(09) Happiness"),
                    lowFrequency: new Value("lowFrequency","(08) Sadness, (06) Tenderness, (05) Fear, (01) Happiness"),
                    //attack
                    fastAttack: new Value("fastAttack","(09) Anger, (05) Happiness, (01) Fear"),
                    slowAttack: new Value("slowAttack","(08) Sadness, (03) Tenderness, (02) Fear, (01) Happiness"),
                    //pause
                    beforeClosure: new Value("beforeClosure",""),
                    afterClosure: new Value("afterClosure",""),
                    // rhythm
                    regularRhythm: new Value("regularRhythm","(02) Happiness, (02) Dignity, (01) Pleasantness, (01) Masterful, (01) Calm"),
                    irregularRhythm: new Value("irregularRhythm","(01) Pleasantness"),
                    complexRhythm: new Value("complexRhythm","(02) Anger"),
                    variedRhythm: new Value("variedRhythm","(01) Happiness"),
                    firmRhythm: new Value("firmRhythm","(02) Sadness, (01) Masterful, (01) Excitement, (01) Dignity"),
                    flowingRhythm: new Value("flowingRhythm","(02) Happiness, (01) Sleepy, (01) Dignity, (01) Calm"),
                    //tempo
                    fast: new Value("fast","(43) Happiness, (24) Anger, (10) Fear, (08) Energy, (06) Pleasantness, (05) Surprise, (02) Masterful, (01) Expectation, (01) Dignity"),
                    moderate: new Value("moderate","(04) Happiness, (03) Fear, (01) Tenderness, (01) Anger"),
                    slow: new Value("slow","(40) Sadness, (14) Tenderness, (08) Dignity, (04) Fear, (03) Calm, (01) Sleepy, (01) Excitement, (01) Disgust, (01) Boredom"),
                    //tempoVariation
                    largeTempoVariation: new Value("largeTempoVariation","(09) Fear, (03) Tenderness, (03) Sadness"),
                    mediumTempoVariation: new Value("mediumTempoVariation","(03) Sadness, (03) Anger, (01) Happiness"),
                    lowTempoVariation: new Value("lowTempoVariation","(07) Happiness, (02) Anger, (01) Tenderness, (01) Sadness"),
                    //noteDensity
                    highDensity: new Value("highDensity","(01) Happiness, (01) Fear"),
                    lowDensity: new Value("lowDensity","(01) Sadness"),
                    //pattern
                    repetition: new Value("repetition","(01) Expectation"),
                    sequence: new Value("sequence",""),
                    //noteDurationContrast
                    sharpBig: new Value("sharpBig","(05) Anger, (03) Happiness, (02) Fear"),
                    softSmall: new Value("softSmall","(05) Tenderness, (04) Sadness, (02) Happiness")
                };
    
                //PROPERTIES
                this.properties = {
                    // visual properties
                    form: new Property("form", "line,point,circle,oval,triangle,square,rectangle,pentagon,star,diamond,infinity"),
                    regularity: new Property("regularity", "regular,irregular"),
                    curve: new Property("curve", "wavy,straight"),
                    edge: new Property("edge", "sharp,round"),
                    closure: new Property("closure", "open,close"),
                    thickness: new Property("thickness", "thin,mediumThick,thick"),
                    size: new Property("size", "big,mediumSize,small"),
                    hue: new Property("hue", "red,orange,yellow,green,blue,violet,purple,pink,brown,grey,black,white,multicolor"),
                    brightness: new Property("brightness", "bright,dark"),
                    saturation: new Property("saturation", "vivid,low"),
                    transparency: new Property("transparency", ""),
                    complexity: new Property("complexity", "simple,complex,clear,crowded"),
                    balance: new Property("balance", "balanced,imbalanced"),
                    density: new Property("density", "dense,sparse"),
                    volume: new Property("volume", "solid,liquid"),
                    distribution: new Property("distribution", "structured,disordered"),
                    texture: new Property("texture", "soft,rough,sticky,nonSticky"),
                    expansion: new Property("expansion", "contractive,expanding"),
                    movement: new Property("movement", "fastMotion,slowMotion,staticMotion"),
                    movementDirection: new Property("movementDirection", "up,down"),
                    symmetry: new Property("symmetry", "symmetric,asymmetric"),
                    repetition: new Property("repetition", "repete,singularity"),
                    rotation: new Property("rotation", "angle"),
                    cluster: new Property("cluster", "attract,repel"),
                    depth: new Property("depth", "background,foreground"),
                    //diffusion: new Property("diffusion", ""),
                    distance: new Property("distance", ""),
                    //influence: new Property("influence", ""),
                    variation: new Property("variation", "contrasting,subtle"),
                    overlap: new Property("overlap", "intersect,parallelism"),
                    position: new Property("position", "horizontal,vertical"),
                    weight: new Property("weight", "light,heavy"),
                    /////////////////////////////////
                    // music properties
                    harmonySonance: new Property("harmonySonance","consonant,dissonant"),
                    harmonyIntervals: new Property("harmonyIntervals","highInterval,lowInterval"),
                    melodyIntervals: new Property("melodyIntervals","m2,m7,M3,M6,P4,P5,Ag4,unison,octave,tritones"),
                    melodyPitchRange: new Property("melodyPitchRange","widePitchRange,narrowPitchRange"),
                    pitch: new Property("pitch","highPitch,lowPitch"),
                    direction: new Property("direction","ascending,descending"),
                    motion: new Property("motion","stepwise,intervallicLeaps,septwiseLeaps"),
                    pitchVariation: new Property("pitchVariation","largePitchVariation,mediumPitchVariation,smallPitchVariation"),
                    mode: new Property("mode","major,minor"),
                    tonality: new Property("tonality","atonal,tonal"),
                    amplitude: new Property("amplitude","roundAmplitude,sharpAmplitude"),
                    articulation: new Property("articulation","staccato,legato"),
                    articulationVariation: new Property("articulationVariation","largeArticulationVariation,mediumArticulationVariation,smallArticulationVariation"),
                    loudness: new Property("loudness","loud,moderate,softLoudness"),
                    loudnessVariation: new Property("loudnessVariation","largeLoudnessVaration,moderateLoudnessVariation,smallLoudnessVariation,rapidChanges,fewChanges"),
                    vibrato: new Property("vibrato","fastVibrato,slowVibrato,irregularVibrato,largeExtent,smallExtent"),
                    timbre: new Property("timbre","manyHarmonics,fewHarmonics,sharpTimbre,softTimbre"),
                    highFrequencyEnergy: new Property("highFrequencyEnergy","highFrequency,mediumFrequency,lowFrequency"),
                    attack: new Property("attack","fastAttack,slowAttack"),
                    pause: new Property("pause","afterClosure,beforeClosure"),
                    rhythm: new Property("rhythm","regularRhythm,irregularRhythm,complexRhythm,variedRhythm,firmRhythm,flowingRhythm"),
                    tempo: new Property("tempo","fast,moderate,slow"),
                    tempoVariation: new Property("tempoVariation","largeTempoVariation,mediumTempoVariation,lowTempoVariation"),
                    noteDensity: new Property("noteDensity","highDensity,lowDensity"),
                    pattern: new Property("pattern","repetition,sequence"),
                    noteDurationContrast: new Property("noteDurationContrast","sharpBig,softSmall")
                };
    
                //LEVELS
                this.levels = {
                    //image
                    low: new Level("low", "form,regularity,curve,edge,closure,thickness,size,hue,brightness,saturation,transparency"),
                    high: new Level("high", "balance,complexity,density,distribution,position,texture,volume"),
                    manipulations: new Level("manipulations","expansion,movement,movementDirection,symmetry,repetition,rotation"),
                    relations: new Level("relations","cluster,depth,distance,variation,overlap,weight"),
                    /////////////////////////////////
                    //music
                    harmonyMelody: new Level("harmonyMelody","harmonySonance,harmonyIntervals,melodyIntervals,melodyPitchRange,pitch,direction,motion,pitchVariation,mode,tonality"),
                    dynamics: new Level("dynamics","amplitude,articulation,articulationVariation,loudness,loudnessVariation,vibrato"),
                    texture: new Level("texture","timbre,highFrequencyEnergy,attack"),
                    temporalOrganization: new Level("temporalOrganization","pause,rhythm,tempo,tempoVariation,noteDensity,pattern,noteDurationContrast")
                };
    
                //DOMAINS
                this.domains = {
                    image: new Domain(this, "image", "high,low,manipulations,relations"),
                    music: new Domain(this, "music", "harmonyMelody,dynamics,texture,temporalOrganization")
                };
            }
    
            get(category, filterName) { //p.e. "emotions" of "highLevel"
                let emos = {};
                if (!filterName) emos = this[category];
                else
                    for (let v of Object.values(this[category]))
                        if (v.hasParent(filterName)) emos[v.label] = v;
                        else if (v.hasChild(filterName)) emos[v.label] = v;
                return emos;
            }
    
            getEmotion(emotionName) {
                return this.emotions[emotionName];
            }
    
            getValue(propertyName) {
                return this.properties[propertyName];
            }
    
            getProperty(propertyName) {
                return this.properties[propertyName];
            }
    
            getLevel(levelName) {
                return this.levels[levelName];
            }
    
            getDomain(domainName) {
                return this.domains[domainName];
            }
    
            getCategory(categoryName) {
                return this[categoryName];
            }
    
            getEmotions(filterName) {
                return filterName ? this.get("emotions", filterName) : this.emotions;
            }
    
            getValues(filterName) {
                return filterName ? this.get("properties", filterName) : this.properties;
            }
    
            getProperties(filterName) {
                return filterName ? this.get("properties", filterName) : this.properties;
            }
    
            getLevels(filterName) {
                return filterName ? this.get("levels", filterName) : this.levels;
            }
    
            getDomains(filterName) {
                return filterName ? this.get("domains", filterName) : this.domains;
            }
    
        },
        emotionsDecipher = ed;
        */